#include "foxcvs.h"
#include "icons.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <dirent.h>
#include "inputdlg.h"
#include "addprjdlg.h"
#include "branchdlg.h"


struct FileData
{
	FXString status;
	FXString version;
	FXString directory;
};
		

#ifndef PIPE_BUF
#define PIPE_BUF 5120
#endif

// Map
FXDEFMAP(MainWindow) MainWindowMap[]={
	FXMAPFUNC(SEL_COMMAND,	MainWindow::ID_ABOUT,	   	MainWindow::onCmdAbout),
	FXMAPFUNC(SEL_COMMAND,	MainWindow::ID_ADD,			MainWindow::onCmdAdd),
	FXMAPFUNC(SEL_UPDATE,	MainWindow::ID_ADD,			MainWindow::onUpdatePrj),
	FXMAPFUNC(SEL_COMMAND,	MainWindow::ID_DELETE,		MainWindow::onCmdDelete),
	FXMAPFUNC(SEL_UPDATE,	MainWindow::ID_DELETE,		MainWindow::onUpdate),
	FXMAPFUNC(SEL_COMMAND,	MainWindow::ID_STATUS,		MainWindow::onCmdStatus),
	FXMAPFUNC(SEL_UPDATE,	MainWindow::ID_STATUS,		MainWindow::onUpdate),
	FXMAPFUNC(SEL_COMMAND,	MainWindow::ID_LOG,			MainWindow::onCmdLog),
	FXMAPFUNC(SEL_UPDATE,	MainWindow::ID_LOG,			MainWindow::onUpdate),
	FXMAPFUNC(SEL_COMMAND,	MainWindow::ID_DIFF,		MainWindow::onCmdDiff),
	FXMAPFUNC(SEL_UPDATE,	MainWindow::ID_DIFF,		MainWindow::onUpdate),
	FXMAPFUNC(SEL_COMMAND,	MainWindow::ID_COMMIT,		MainWindow::onCmdCommit),
	FXMAPFUNC(SEL_UPDATE,	MainWindow::ID_COMMIT,		MainWindow::onUpdate),
	FXMAPFUNC(SEL_COMMAND,	MainWindow::ID_WATCH,		MainWindow::onCmdWatch),
	FXMAPFUNC(SEL_UPDATE,	MainWindow::ID_WATCH,		MainWindow::onUpdate),
	FXMAPFUNC(SEL_COMMAND,	MainWindow::ID_BRANCH,		MainWindow::onCmdBranch),
	FXMAPFUNC(SEL_UPDATE,	MainWindow::ID_BRANCH,		MainWindow::onUpdatePrj),
	FXMAPFUNC(SEL_UPDATE,	MainWindow::ID_UPDATE,		MainWindow::onUpdatePrj),
	FXMAPFUNC(SEL_COMMAND,	MainWindow::ID_TAG,			MainWindow::onCmdTag),
	FXMAPFUNC(SEL_UPDATE,	MainWindow::ID_TAG,			MainWindow::onUpdatePrj),
	FXMAPFUNC(SEL_COMMAND,	MainWindow::ID_PRJ_NEW,		MainWindow::onCmdNewPrj),
	FXMAPFUNC(SEL_CLICKED,	MainWindow::ID_FOLDER_TREE,	MainWindow::onCmdClkTree),
};

// Object implementation
FXIMPLEMENT(MainWindow,FXMainWindow,MainWindowMap,ARRAYNUMBER(MainWindowMap))

/*******************************************************************************/
  
MainWindow::MainWindow(FXApp* a) : 
		FXMainWindow(a,"FOX CVS Client",NULL,NULL,DECOR_ALL,50,50,800,600,0,0)
{
	prjdata=NULL;
	folderOpen=new FXGIFIcon(a,minifolderopen);
	folderClosed=new FXGIFIcon(a,minifolderclosed);
	newIcon=new FXGIFIcon(a,new_gif);
	saveIcon=new FXGIFIcon(a,save_gif);
	saveasIcon=new FXGIFIcon(a,saveas_gif);
	openIcon=new FXGIFIcon(a,open_gif);
	deleteIcon=new FXGIFIcon(a,delete_gif);
	logIcon=new FXGIFIcon(a,log_gif);
	statusIcon=new FXGIFIcon(a,status_gif);
	diffIcon=new FXGIFIcon(a,diff_gif);
	watchIcon=new FXGIFIcon(a,watch_gif);
	tagIcon=new FXGIFIcon(a,tag_gif);
	branchIcon=new FXGIFIcon(a,branch_gif);
	
	// Tooltip
	new FXTooltip(a);
	
	// Make menu bar
	FXToolbarShell* dragshell1=new FXToolbarShell(this,0);
	FXMenubar *menubar=new FXMenubar(this,dragshell1,LAYOUT_SIDE_TOP|
					LAYOUT_FILL_X|FRAME_RAISED);
	new FXToolbarGrip(menubar,this,FXToolbar::ID_TOOLBARGRIP);
  
	// Tool bar
	FXToolbarShell* dragshell2=new FXToolbarShell(this,0);
	FXToolbar *toolbar=new FXToolbar(this,dragshell2,LAYOUT_SIDE_TOP|
			LAYOUT_FILL_X|PACK_UNIFORM_WIDTH|PACK_UNIFORM_HEIGHT|FRAME_RAISED);
	new FXToolbarGrip(toolbar,this,FXToolbar::ID_TOOLBARGRIP);

	
	// Status bar
	new FXStatusbar(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER);
  
	FXSplitter *splitter2=new FXSplitter(this,SPLITTER_VERTICAL|
					LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_TRACKING);
	FXVerticalFrame *content=new FXVerticalFrame(splitter2,FRAME_NONE|
					LAYOUT_FILL_X|LAYOUT_FILL_Y,
					0,0,0,350, 0,0,0,0);
	FXSplitter *splitter=new FXSplitter(content,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|
					LAYOUT_FILL_Y|SPLITTER_TRACKING);
	
	FXVerticalFrame *folderframe=new FXVerticalFrame(splitter,FRAME_SUNKEN|FRAME_THICK|
					LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,145,0, 0,0,0,0);
	m_folders = new FXTreeList(folderframe,0,this,ID_FOLDER_TREE,
    				TREELIST_ROOT_BOXES|TREELIST_SHOWS_BOXES|TREELIST_SHOWS_LINES|
					LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP
					|LAYOUT_RIGHT);
	
	FXVerticalFrame *messageframe=new FXVerticalFrame(splitter,FRAME_SUNKEN|FRAME_THICK|
					LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0);
	m_files=new FXIconList(messageframe,this,ID_FILE_LIST,LAYOUT_FILL_X|LAYOUT_FILL_Y|
							ICONLIST_DETAILED);
 
	FXVerticalFrame *previewframe=new FXVerticalFrame(splitter2,FRAME_SUNKEN|FRAME_THICK|
					LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0);
	m_cmdbox = new FXText(previewframe,NULL,0,
					LAYOUT_FILL_X|LAYOUT_FILL_Y);

   	// File Menu
	filemenu=new FXMenuPane(this);
	new FXMenuCommand(filemenu,"Add\t\tAdd files to the repository.",
			newIcon,this,ID_ADD);
	new FXMenuCommand(filemenu,
			"Delete\t\tDelete files from the repository.",
			deleteIcon,this,ID_DELETE);
	new FXMenuCommand(filemenu,"Status\t\tShow selected file's status.",
			statusIcon,this,ID_STATUS);
	new FXMenuCommand(filemenu,"Log\t\tShow selected file's log.",
			logIcon,this,ID_LOG);
	new FXMenuCommand(filemenu,"Commit\t\tCommit changes to repository.",
			saveIcon,this,ID_COMMIT);
	new FXMenuCommand(filemenu,"Diff\t\tDifference from repository to local file.",
			diffIcon,this,ID_DIFF);
	new FXMenuCommand(filemenu,"Update\t\tBring local in sync with repository.",
			saveasIcon,this,ID_UPDATE);
	new FXMenuCommand(filemenu,"Tag\t\tAdd a symbolic tag to this project.",
			tagIcon,this,ID_TAG);
	new FXMenuCommand(filemenu,"Branch\t\tBranch off a new developement path",
			branchIcon,this,ID_BRANCH);
	new FXMenuCommand(filemenu,"Watch\t\tSet a file watch.",
			watchIcon,this,ID_WATCH);
	new FXMenuSeparator(filemenu);
	new FXMenuCommand(filemenu,"&Quit\tCtl-Q\tQuit foxcvs",NULL,getApp(),FXApp::ID_QUIT);
	new FXMenuTitle(menubar,"&File",NULL,filemenu);

	FXMenuPane *prjmenu=new FXMenuPane(this);
	new FXMenuCommand(prjmenu,"&New",NULL,this,ID_PRJ_NEW);
	new FXMenuCommand(prjmenu,"&Edit",NULL,this,ID_PRJ_EDIT);
	new FXMenuCommand(prjmenu,"&Delete",NULL,this,ID_PRJ_DELETE);
	new FXMenuTitle(menubar,"&Project",NULL,prjmenu);
	
	FXMenuPane *cvsmenu=new FXMenuPane(this);
	new FXMenuCommand(cvsmenu,"&Import",NULL,this,ID_IMPORT);
	new FXMenuCommand(cvsmenu,"&Checkout",NULL,this,ID_CHECKOUT);
	new FXMenuCommand(cvsmenu,"&Release",NULL,this,ID_RELEASE);
	new FXMenuTitle(menubar,"&CVS",NULL,cvsmenu);
	
	helpmenu=new FXMenuPane(this);
	new FXMenuCommand(helpmenu,"&About",NULL,this,ID_ABOUT);
	new FXMenuTitle(menubar,"&Help",NULL,helpmenu);
	
	//Toolbar
	new FXButton(toolbar,"Add\tAdd Files\tAdd files to the repository.",
			newIcon,this,ID_ADD,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
	new FXButton(toolbar,"Delete\tDelete Files\tDelete files from the repository.",
			deleteIcon,this,ID_DELETE,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
	new FXButton(toolbar,"Status\tStatus\tShow selected file's status.",
			statusIcon,this,ID_STATUS,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
	new FXButton(toolbar,"Log\tLog\tShow selected file's log.",
			logIcon,this,ID_LOG,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
	new FXButton(toolbar,"Commit\tCommit\tCommit changes to repository.",
			saveIcon,this,ID_COMMIT,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
	new FXButton(toolbar,"Diff\tDiff\tDifference from repository to local file.",
			diffIcon,this,ID_DIFF,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
	new FXButton(toolbar,"Update\tUpdate\tBring local in sync with repository.",
			saveasIcon,this,ID_UPDATE,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
	new FXButton(toolbar,"Tag\tTag\tAdd a symbolic tag to this project.",
			tagIcon,this,ID_TAG,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
	new FXButton(toolbar,"Branch\tBranch\tBranch off a new developement path",
			branchIcon,this,ID_BRANCH,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
	new FXButton(toolbar,"Watch\tWatch\tSet a file watch.",
			watchIcon,this,ID_WATCH,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
 
	LoadProjects();

	m_files->appendHeader("File",NULL,100);
	m_files->appendHeader("Status",NULL,150);
	m_files->appendHeader("Revision",NULL,60);
	m_files->appendHeader("Revision Date",NULL,200);
	m_files->appendHeader("Local Directory",NULL,200);

	splitter->setBarSize(1);
	splitter2->setBarSize(1);
}

void MainWindow::LoadProjects()
{
	FXTreeItem *curitem;
	curitem = m_folders->addItemLast(0,"Projects",folderOpen,folderClosed);
	
	FXString strpath;
	char *home;
	if (!(home=getenv("HOME"))) {
		fprintf(stderr,"No HOME env variable!\n");
		return ;			
	}
	strpath = FXString(home) + "/.foxcvsrc";			
	FILE *file;
	if (!(file=fopen(strpath.text(),"r"))) {
		fprintf(stderr,"No file %s\n",strpath.text());
		return ;	
	}
	//TODO
	//Arbitrary size for max project file size....
	char buffer[5120];
	int nRead = fread(buffer,1,5119,file);
	buffer[nRead] = 0;
	fclose(file);
	
	int i=0;
	char *line,*ptr;
	line = buffer;
	PrjData *prjdata;
	FXString pline;
	while (i<nRead) {
		if (buffer[i] == '\n') {
			buffer[i] = 0;
			//ignore if comment or section header
			if ((strcasecmp(line,"[Projects]") != 0) && (line[0] != '#')) {
				if ((ptr=strchr(line,'|'))) {
					*ptr = 0;
					ptr++;
					prjdata = new PrjData;
					prjdata->folder = ptr;
					m_folders->addItemLast(curitem,line,folderOpen,folderClosed,prjdata);
				}
			}
			i++;
			line = &buffer[i];
		}
		else {
			i++;
		}
	}
	m_folders->expandTree(curitem);
}

// Start
void MainWindow::create(){
	FXMainWindow::create();
	show();
}

long MainWindow::onCmdAbout(FXObject*,FXSelector,void*)
{  
	FXMessageBox about(this,"About foxcvs",
			"foxcvs 0.1\n"
			"FOX CVS Client\n"
			"Davin S. Hills\n"
			"dshills@uswest.net\n"
			"http://www.users.uswest.net/~dshills/linux.html",
			NULL,MBOX_OK|DECOR_TITLE|DECOR_BORDER);
	about.execute(); 
	
	return 1;
}

long MainWindow::onUpdatePrj(FXObject *sender, FXSelector,void*)
{
	FXSelector msg=MKUINT(ID_DISABLE,SEL_COMMAND);
	if (prjdata != NULL) // Project selected
		msg=MKUINT(ID_ENABLE,SEL_COMMAND);
	sender->handle(this,msg,NULL);
	return 1;

}

long MainWindow::onUpdate(FXObject *sender, FXSelector,void*)
{
	FXSelector msg=MKUINT(ID_DISABLE,SEL_COMMAND);
	if (m_files->getCurrentItem() != -1) // Item selected
		msg=MKUINT(ID_ENABLE,SEL_COMMAND);
	sender->handle(this,msg,NULL);
	return 1;
}

long MainWindow::onCmdAdd(FXObject *sender,FXSelector,void*)
{
	// Adds a file to the CVS repository
	FXFileDialog open(this,"Add File");
	if(open.execute()){
		FXString filename,ret,cmd,log;
		filename=open.getFilename();
		cmd = "cvs add " + filename;
		ret = RunCmd(cmd);
		InputDialog inputdlg(getApp());
		inputdlg.setTitle("Log Message");
		if (inputdlg.execute()) {
			log = inputdlg.input->getText();
			cmd = "cvs commit -m \"" + log + "\" " + filename;
			ret += RunCmd(cmd);
			if (ret.empty())
				ret += "Nothing to commit.";
			m_cmdbox->setText(ret);
			RefreshFiles(m_folders->getCurrentItem());
		}
	}
	return 1;
}

long MainWindow::onCmdDelete(FXObject*,FXSelector,void*)
{
	// Deletes a file from the repository AND local!
	int sel = m_files->getCurrentItem();
	if (sel == -1)
		return 1;
	FXString ret,cmd,msg;
	ret = m_files->getItemText(sel);
	CVSStatus cvsstat;
	ParseFileEntry(ret,&cvsstat);

	msg.format("Are you sure want to delete %s and "
			"remove it from the repository?",cvsstat.fname.text());

	FXMessageBox delbox(this,"Confirm Delete",msg,
					NULL,MBOX_YES_NO|DECOR_TITLE|DECOR_BORDER);

	if (delbox.execute() == MBOX_CLICKED_YES) {
		chdir(cvsstat.directory.text()); //Make sure we are in the correct directory
		remove(cvsstat.fname.text());
		cmd = "cvs delete " + cvsstat.fname;
		ret = RunCmd(cmd);
		cmd = "cvs commit -m \"Deleting file.\" " + cvsstat.fname;
		ret += RunCmd(cmd);
		m_cmdbox->setText(ret);
		m_cmdbox->makePositionVisible(ret.length());
		RefreshFiles(m_folders->getCurrentItem());
	}

	return 1;
}

long MainWindow::onCmdStatus(FXObject*,FXSelector,void*)
{
	// Get file status or project status
	int sel = m_files->getCurrentItem();
	if (sel == -1)
		return 1;
	FXString ret,cmd;
	ret = m_files->getItemText(sel);
	CVSStatus cvsstat;
	ParseFileEntry(ret,&cvsstat);
	
	cmd = "cvs status -v " + cvsstat.fname;
	chdir(cvsstat.directory.text());
	ret = RunCmd(cmd);
	m_cmdbox->setText(ret);
	m_cmdbox->makePositionVisible(ret.length());
	return 1;
}

long MainWindow::onCmdLog(FXObject*,FXSelector,void*)
{
	// Get a log for file or project
	int sel = m_files->getCurrentItem();
	if (sel == -1)
		return 1;
	FXString ret,cmd;
	ret = m_files->getItemText(sel);
	CVSStatus cvsstat;
	ParseFileEntry(ret,&cvsstat);
	
	cmd = "cvs log " + cvsstat.fname;
	chdir(cvsstat.directory.text());
	ret = RunCmd(cmd);
	m_cmdbox->setText(ret);
	return 1;
}

long MainWindow::onCmdDiff(FXObject*,FXSelector,void*)
{
	int sel = m_files->getCurrentItem();
	if (sel == -1)
		return 1;
	FXString ret,cmd;
	ret = m_files->getItemText(sel);
	CVSStatus cvsstat;
	ParseFileEntry(ret,&cvsstat);
	
	cmd = "cvs diff " + cvsstat.fname;
	chdir(cvsstat.directory.text());
	ret = RunCmd(cmd);
	if (ret.empty())
		ret = "No difference between local and repository file.";
	m_cmdbox->setText(ret);
	return 1;
}

long MainWindow::onCmdCommit(FXObject*,FXSelector,void*)
{
	int sel = m_files->getCurrentItem();
	if (sel == -1)
		return 1;
	FXString ret,cmd,log;
	ret = m_files->getItemText(sel);
	CVSStatus cvsstat;
	ParseFileEntry(ret,&cvsstat);

	InputDialog inputdlg(getApp());
	inputdlg.setTitle("Log Message");
	if (inputdlg.execute()) {
		log = inputdlg.input->getText();
		cmd = "cvs commit -m \"" + log + "\" " + cvsstat.fname;
		chdir(cvsstat.directory.text());
		ret = RunCmd(cmd);
		if (ret.empty())
			ret = "Nothing to commit.";
		m_cmdbox->setText(ret);
		RefreshFiles(m_folders->getCurrentItem());
	}
	return 1;
}

long MainWindow::onCmdWatch(FXObject*,FXSelector,void*)
{
	int sel = m_files->getCurrentItem();
	if (sel == -1)
		return 1;
	FXString ret,cmd;
	ret = m_files->getItemText(sel);
	CVSStatus cvsstat;
	ParseFileEntry(ret,&cvsstat);
	
	cmd = "cvs watch on " + cvsstat.fname;
	chdir(cvsstat.directory.text());
	ret = RunCmd(cmd);
	m_cmdbox->setText(ret);

	return 1;
}

long MainWindow::onCmdTag(FXObject*,FXSelector,void*)
{
	if (prjdata == NULL)
		return 1;
	FXString ret,cmd,log;
	FXInputDialog id(this,"Project Tag","Tag Name");
	if (id.execute()) {	
		log = id.getText();
		cmd = "cvs tag -c " + log + " 2>&1";
		chdir(prjdata->folder.text());// Goto highest dir in project to tag recursivly
		ret = RunCmd(cmd);
		m_cmdbox->setText(ret);
	}
	return 1;
	
}

long MainWindow::onCmdBranch(FXObject*,FXSelector,void*)
{
	if (prjdata == NULL)
		return 1;
	FXTreeItem *item;
	item = m_folders->getCurrentItem();
	
	FXString cmd,ret;
	BranchDialog bd(getApp());
	if (bd.execute()) {
		cmd = "cvs rtag -b -r " + bd.revname->getText() + " " + 
				bd.bname->getText() + " " + item->getText() + " 2>&1";
		ret = RunCmd(cmd);
		m_cmdbox->setText(ret);
	}

	return 1;
}

long MainWindow::onCmdNewPrj(FXObject*,FXSelector,void*)
{
	AddPrjDlg pd(getApp());
	pd.execute();
	return 1;
}

FXString MainWindow::RunCmd(FXString cmd)
{
	FILE *readfp;
	char buffer[BUFSIZ+1];
	int charread;
	memset(buffer,0,sizeof(buffer));
	readfp = popen(cmd.text(),"r");
	if (readfp) {
		charread = fread(buffer,sizeof(char),BUFSIZ,readfp);
		pclose(readfp);
	}
	FXString ret(buffer);
	return buffer;
}

long MainWindow::onCmdClkTree(FXObject*, FXSelector, void *ptr)
{
	RefreshFiles((FXTreeItem*)ptr);
	return 1;
}

void MainWindow::RefreshFiles(FXTreeItem *item) 
{
	m_files->clearItems();
	if (!(prjdata = (PrjData*)item->getData()))
			return ;
	ScanDir((char*)prjdata->folder.text());
	m_files->setCurrentItem(-1);
}

void MainWindow::ScanDir(char *directory)
{
	fprintf(stderr,"Scanning: %s\n",directory);

	FXString fileentry,cmd,ret;
	DIR *dp;
	dirent *entry;
	struct stat statbuf;
	CVSStatus cvsstat;
	
	chdir(directory); //Change to directory for cvs
	dp = opendir(directory);
	while ((entry = readdir(dp))) {
		memset(&statbuf,0,sizeof(statbuf));
		stat(entry->d_name,&statbuf);
		if (S_ISDIR(statbuf.st_mode)) { //directory
			//ignore . .. CVS directories
			if ((strcmp(".",entry->d_name) != 0) && (strcmp("..",entry->d_name) != 0) &&
					(strcmp("CVS",entry->d_name) != 0)) {
				fileentry = FXString(directory) + "/" + entry->d_name;
				ScanDir((char*)fileentry.text());
			}
		}
		else {
			//Normal file get cvs status
			cmd = FXString("cvs status ") + entry->d_name + " 2>&1";
			ret = RunCmd(cmd);
			cvsstat.clear();
			if (ParseCVSStatus(ret,&cvsstat)) {
				if (cvsstat.status != "Unknown") { //only add files from rep
					fileentry = FXString(entry->d_name) + 
								"\t" + cvsstat.status +
								"\t" + cvsstat.workRev +
								"\t" + cvsstat.revDate +
								"\t" + directory;
					m_files->appendItem(fileentry);
				}
			}
		}
	}
	closedir(dp);
}

void MainWindow::UpdateStatus(int item)
{
	CVSStatus cvsstat;
	FXString ret,cmd,fileentry,fname;
	int pos;
	ret = m_files->getItemText(item);
	pos = ret.findf("\t");
	fname = ret.left(pos);
	cmd = "cvs status " + fname + " 2>&1";
	ret = RunCmd(cmd);
	if (ParseCVSStatus(ret,&cvsstat)) {
		fileentry = FXString() + fname +
								"\t" + cvsstat.status +
								"\t" + cvsstat.workRev +
								"\t" + cvsstat.revDate; 
		m_files->replaceItem(item,fileentry);
	}
}

void MainWindow::ParseFileEntry(FXString entry, CVSStatus *cvsstat)
{
	cvsstat->fname = strtok((char*)entry.text(),"\t");
	cvsstat->status = strtok(NULL,"\t");
	cvsstat->workRev = strtok(NULL,"\t");
	cvsstat->revDate = strtok(NULL,"\t");
	cvsstat->directory = strtok(NULL,"\t");
}

bool MainWindow::ParseCVSStatus(FXString ret, CVSStatus *cvsstat)
{
	int pos;

	//Status
	pos = ret.findf("Status:");
	if (pos == -1)
		return false;
	pos += 8; //Jump past "Status: "
	while (ret[pos] != '\n')
		cvsstat->status += ret[pos++];
	//Working revision
	pos = ret.findf("Working revision:\t");
	if (pos != -1) {
		pos += 18; 
		while (ret[pos] != '\t')
			cvsstat->workRev += ret[pos++];
		pos++;
		while (ret[pos] != '\n')
			cvsstat->revDate += ret[pos++];
	}

	return true;
}
